@@ -1,7 +1,6 @@
 import unittest
 import tsaugmentation as tsag
 from htsmodels.models.mint import MinT
-import shutil
 
 
 class TestModel(unittest.TestCase):
@@ -10,13 +9,8 @@ class TestModel(unittest.TestCase):
         self.data = tsag.preprocessing.PreprocessDatasets('tourism', test_size=228*10).apply_preprocess()
         self.n = self.data['predict']['n']
         self.s = self.data['train']['s']
-        shutil.rmtree("./data/original_datasets")
         self.mint = MinT(dataset='tourism', groups=self.data)
 
-    @classmethod
-    def tearDownClass(cls):
-        shutil.rmtree("./results")
-
     def test_correct_train(self):
         model = self.mint.train()
         self.assertIsNotNone(model)
@@ -24,12 +18,7 @@ class TestModel(unittest.TestCase):
     def test_results_interval(self):
         forecasts = self.mint.train()
         results = self.mint.results(forecasts)
-        res = self.mint.metrics(results)
-        self.mint.store_metrics(res)
-        self.assertLess(res['mase']['bottom'], 2.7)
-        self.assertLess(res['CRPS']['bottom_ind'][0], 10)
-
-    def test_predict_shape(self):
-        forecasts = self.mint.train()
-        df_results = self.mint.results(forecasts)
-        self.assertTrue(df_results.shape == (1872, 11))
+        pred_mean, pred_std = self.mint.predict(results)
+        res = self.mint.metrics(pred_mean, pred_std)
+        self.assertLess(res['mase']['bottom'], 2.2)
+        self.assertLess(res['CRPS']['bottom_ind'][0], 100)
