@@ -1,6 +1,7 @@
 import numpy as np
 from sklearn.metrics import mean_squared_error
 import properscoring as ps
+from sktime.performance_metrics.forecasting import MeanAbsoluteScaledError
 
 
 class CalculateResultsBase:
@@ -15,7 +16,7 @@ class CalculateResultsBase:
 
     """
 
-    def __init__(self, groups, samples=500):
+    def __init__(self, groups, dataset):
         self.groups = groups
         self.seas = self.groups["seasonality"]
         self.h = self.groups["h"]
@@ -25,39 +26,11 @@ class CalculateResultsBase:
         self.errs = ["mase", "rmse", "CRPS"]
         self.levels = list(self.groups["train"]["groups_names"].keys())
         self.levels.extend(("bottom", "total"))
-        self.samples = samples
-
-    def mase(self, y, f):
-        """Calculates the Mean Absolute Scaled Error
-
-        Args:
-            param y (array): original series values shape=[n,s]
-            param f (array): predictions shape=[h,s]
-
-        Returns:
-            MASE (array): shape=[s,]
-
-        """
-        return (
-            (self.n - self.seas)
-            / self.h
-            * (
-                np.sum(np.abs(y[self.n - self.h : self.n, :] - f), axis=0)
-                / np.sum(
-                    np.abs(y[self.seas : self.n, :] - y[: self.n - self.seas, :]),
-                    axis=0,
-                )
-            )
-        )
+        self.dataset = dataset
+        self.mase = MeanAbsoluteScaledError(multioutput="raw_values")
 
     def calculate_metrics_for_individual_group(
-        self,
-        group_name,
-        y,
-        predictions_mean,
-        error_metrics,
-        predictions_sample=np.zeros((1,)),
-        predictions_std=None,
+        self, group_name, y, predictions_mean, predictions_std, error_metrics
     ):
         """Calculates the main metrics for each group
 
@@ -73,56 +46,31 @@ class CalculateResultsBase:
             error (obj): contains both the error metric for each individual series of each group and the average
 
         """
-        y_p = y[self.n - self.h : self.n, :]
-        if predictions_sample.any():
-            predictions_sample = predictions_sample[self.n - self.h : self.n, :, :]
-        n_s = y_p.shape[1]
+
+        y_true = y[-self.h :, :]
+        y_train = y[: -self.h, :]
+        f = predictions_mean[-self.h :]
+        f_std = predictions_std[-self.h :]
         error_metrics["mase"][f"{group_name}_ind"] = np.round(
-            self.mase(y=y, f=predictions_mean), 3
+            self.mase(y_true=y_true, y_pred=f, y_train=y_train, sp=self.seas), 3
         )
         error_metrics["mase"][f"{group_name}"] = np.round(
             np.mean(error_metrics["mase"][f"{group_name}_ind"]), 3
         )
         error_metrics["rmse"][f"{group_name}_ind"] = np.round(
-            mean_squared_error(
-                y_p, predictions_mean, squared=False, multioutput="raw_values"
-            ),
+            mean_squared_error(y_true, f, squared=False, multioutput="raw_values"),
             3,
         )
         error_metrics["rmse"][f"{group_name}"] = np.round(
             np.mean(error_metrics["rmse"][f"{group_name}_ind"]), 3
         )
 
-        if predictions_std is not None:
-            error_metrics["CRPS"][f"{group_name}"] = ps.crps_ensemble(
-                y_p,
-                np.transpose(
-                    np.random.normal(
-                        loc=predictions_mean,
-                        scale=predictions_std,
-                        size=(self.samples, self.h, n_s),
-                    ),
-                    (1, 2, 0),
-                ),
-            ).mean()
-            error_metrics["CRPS"][f"{group_name}_ind"] = ps.crps_ensemble(
-                y_p,
-                np.transpose(
-                    np.random.normal(
-                        loc=predictions_mean,
-                        scale=predictions_std,
-                        size=(self.samples, self.h, n_s),
-                    ),
-                    (1, 2, 0),
-                ),
-            ).mean(axis=0)
-        else:
-            error_metrics["CRPS"][f"{group_name}"] = ps.crps_ensemble(
-                y_p, predictions_sample
-            ).mean()
-            error_metrics["CRPS"][f"{group_name}_ind"] = ps.crps_ensemble(
-                y_p, predictions_sample
-            ).mean(axis=0)
+        error_metrics["CRPS"][f"{group_name}"] = ps.crps_gaussian(
+            y_true, f, f_std
+        ).mean()
+        error_metrics["CRPS"][f"{group_name}_ind"] = ps.crps_gaussian(
+            y_true, f, f_std
+        ).mean(axis=0)
 
         return error_metrics
 
@@ -142,17 +90,12 @@ class CalculateResultsBottomUp(CalculateResultsBase):
         all the information regarding the different groups
     """
 
-    def __init__(
-        self, pred_samples, groups, store_prediction_samples, store_prediction_points
-    ):
-        super().__init__(groups=groups)
-        self.pred_mean = np.mean(pred_samples, axis=2).reshape(self.n, self.s)[
-            self.n - self.h : self.n, :
-        ]
-        self.pred_mean_complete = np.mean(pred_samples, axis=2).reshape(self.n, self.s)
-        self.pred_samples = pred_samples
-        self.store_prediction_samples = store_prediction_samples
-        self.store_prediction_points = store_prediction_points
+    def __init__(self, groups, dataset, predictions_mean, predictions_std):
+        super().__init__(groups=groups, dataset=dataset)
+        self.groups_names = list(self.groups["predict"]["groups_names"].keys())
+        self.n_samples = groups["predict"]["n"]
+        self.predictions_mean = predictions_mean
+        self.predictions_std = predictions_std
 
     def compute_error_for_every_group(self, error_metrics):
         """Computes the error metrics for all the groups
@@ -162,44 +105,38 @@ class CalculateResultsBottomUp(CalculateResultsBase):
                          - contains all the predictions for all the groups
 
         """
-        idx_dict_new = dict()
+        group_element_active = dict()
         for group in list(self.groups["predict"]["groups_names"].keys()):
-            y_g = np.zeros(
-                (
-                    self.groups["predict"]["n"],
-                    self.groups["predict"]["groups_names"][group].shape[0],
-                )
-            )
-            f_g = np.zeros(
-                (self.h, self.groups["predict"]["groups_names"][group].shape[0])
-            )
-            s_g = np.zeros(
-                (
-                    self.n,
-                    self.groups["predict"]["groups_names"][group].shape[0],
-                    self.pred_samples.shape[2],
-                )
-            )
+            n_elements_group = self.groups["predict"]["groups_names"][group].shape[0]
+            group_elements = self.groups["predict"]["groups_names"][group]
+            groups_idx = self.groups["predict"]["groups_idx"][group]
 
-            for idx, name in enumerate(self.groups["predict"]["groups_names"][group]):
-                idx_dict_new[name] = np.where(
-                    self.groups["predict"]["groups_idx"][group] == idx, 1, 0
+            y_g = np.zeros((self.n_samples, n_elements_group))
+            mean_g = np.zeros((self.n_samples, n_elements_group))
+            std_g = np.zeros((self.n_samples, n_elements_group))
+
+            for group_idx, element_name in enumerate(group_elements):
+                group_element_active[element_name] = np.where(
+                    groups_idx == group_idx, 1, 0
                 ).reshape((1, -1))
 
-                y_g[:, idx] = np.sum(idx_dict_new[name] * self.y_f, axis=1)
-                f_g[:, idx] = np.sum(idx_dict_new[name] * self.pred_mean, axis=1)
-                s_g[:, idx, :] = np.sum(
-                    idx_dict_new[name][:, :, np.newaxis] * self.pred_samples, axis=1
+                y_g[:, group_idx] = np.sum(
+                    group_element_active[element_name] * self.y_f, axis=1
+                )
+                mean_g[:, group_idx] = np.sum(
+                    group_element_active[element_name] * self.predictions_mean, axis=1
+                )
+                std_g[:, group_idx] = np.sum(
+                    group_element_active[element_name] * self.predictions_mean, axis=1
                 )
-                if self.store_prediction_points:
-                    error_metrics["predictions"]["points"][name] = f_g[:, idx]
-                if self.store_prediction_samples:
-                    error_metrics["predictions"]["samples"][name] = s_g[:, idx, :]
 
             error_metrics = self.calculate_metrics_for_individual_group(
-                group, y_g, f_g, error_metrics, predictions_sample=s_g
+                group_name=group,
+                y=y_g,
+                predictions_mean=mean_g,
+                predictions_std=std_g,
+                error_metrics=error_metrics,
             )
-
         return error_metrics
 
     def bottom_up(self, level, error_metrics):
@@ -211,35 +148,25 @@ class CalculateResultsBottomUp(CalculateResultsBase):
         """
         if level == "bottom":
             error_metrics = self.calculate_metrics_for_individual_group(
-                level,
-                self.y_f,
-                self.pred_mean,
-                error_metrics,
-                predictions_sample=self.pred_samples,
+                group_name=level,
+                y=self.y_f,
+                predictions_mean=self.predictions_mean,
+                predictions_std=self.predictions_std,
+                error_metrics=error_metrics,
             )
-            if self.store_prediction_points:
-                error_metrics["predictions"]["points"]["bottom"] = self.pred_mean
-            if self.store_prediction_samples:
-                error_metrics["predictions"]["samples"]["bottom"] = self.pred_samples
         elif level == "total":
+            np.sqrt(np.sum(self.predictions_std**2, axis=1)).reshape(-1, 1)
             error_metrics = self.calculate_metrics_for_individual_group(
-                level,
-                np.sum(self.y_f, axis=1).reshape(-1, 1),
-                np.sum(self.pred_mean, axis=1).reshape(-1, 1),
-                error_metrics,
-                predictions_sample=np.sum(self.pred_samples, axis=1).reshape(
-                    (self.n, 1, -1)
-                ),
+                group_name=level,
+                y=np.sum(self.y_f, axis=1).reshape(-1, 1),
+                predictions_mean=np.sum(self.predictions_mean, axis=1).reshape(-1, 1),
+                # The variance of the resulting distribution will be the sum
+                # of the variances of the original Gaussian distributions
+                predictions_std=np.sqrt(
+                    np.sum(self.predictions_std**2, axis=1)
+                ).reshape(-1, 1),
+                error_metrics=error_metrics,
             )
-
-            if self.store_prediction_points:
-                error_metrics["predictions"]["points"]["total"] = np.sum(
-                    self.pred_mean, axis=1
-                )
-            if self.store_prediction_samples:
-                error_metrics["predictions"]["samples"]["total"] = np.sum(
-                    self.pred_samples, axis=1
-                )
         elif level == "groups":
             self.compute_error_for_every_group(error_metrics)
 
@@ -257,12 +184,6 @@ class CalculateResultsBottomUp(CalculateResultsBase):
         error_metrics["mase"] = {}
         error_metrics["rmse"] = {}
         error_metrics["CRPS"] = {}
-        if self.store_prediction_points or self.store_prediction_samples:
-            error_metrics["predictions"] = {}
-        if self.store_prediction_points:
-            error_metrics["predictions"]["points"] = {}
-        if self.store_prediction_samples:
-            error_metrics["predictions"]["samples"] = {}
 
         error_metrics = self.bottom_up("bottom", error_metrics)
         error_metrics = self.bottom_up("total", error_metrics)
@@ -300,13 +221,9 @@ class CalculateResultsMint(CalculateResultsBase):
         dictionary with all the information regarding the different groups
     """
 
-    def __init__(
-        self, df_results_mint, groups, store_prediction_samples, store_prediction_points
-    ):
-        super().__init__(groups=groups)
+    def __init__(self, df_results_mint, groups, dataset):
+        super().__init__(groups=groups, dataset=dataset)
         self.df_results_mint = df_results_mint
-        self.store_prediction_samples = store_prediction_samples
-        self.store_prediction_points = store_prediction_points
 
     def compute_error_for_every_group(self, error_metrics):
         """Computes the error metrics for all the groups
@@ -358,17 +275,9 @@ class CalculateResultsMint(CalculateResultsBase):
                         .all(axis=1)
                     ]["std"]
                 ).reshape((self.h,))
-                if self.store_prediction_points:
-                    error_metrics["predictions"]["points"][name] = f_g
-                if self.store_prediction_samples:
-                    error_metrics["predictions"]["samples"][name] = np.random.normal(
-                        loc=f_g[:, idx],
-                        scale=std_g[:, idx],
-                        size=(self.samples, self.h),
-                    ).reshape((self.h, -1))
 
             error_metrics = self.calculate_metrics_for_individual_group(
-                group, y_g, f_g, error_metrics, predictions_std=std_g
+                group, y_g, f_g, std_g, error_metrics
             )
 
         return error_metrics
@@ -404,20 +313,10 @@ class CalculateResultsMint(CalculateResultsBase):
                 level,
                 self.y_f,
                 pred_samples_mean,
+                pred_samples_std,
                 error_metrics,
-                predictions_std=pred_samples_std,
             )
-            if self.store_prediction_points:
-                error_metrics["predictions"]["points"]["bottom"] = pred_samples_mean
-            if self.store_prediction_samples:
-                error_metrics["predictions"]["samples"]["bottom"] = np.transpose(
-                    np.random.normal(
-                        loc=pred_samples_mean,
-                        scale=pred_samples_std,
-                        size=(self.samples, self.h, self.s),
-                    ),
-                    (1, 2, 0),
-                )
+
         elif level == "total":
             n_s = 1
             pred_samples_mean = (
@@ -447,19 +346,9 @@ class CalculateResultsMint(CalculateResultsBase):
                 level,
                 np.sum(self.y_f, axis=1).reshape(-1, 1),
                 pred_samples_mean,
+                pred_samples_std,
                 error_metrics,
-                predictions_std=pred_samples_std,
             )
-            if self.store_prediction_points:
-                error_metrics["predictions"]["points"]["total"] = np.squeeze(
-                    pred_samples_mean
-                )
-            if self.store_prediction_samples:
-                error_metrics["predictions"]["samples"]["total"] = np.random.normal(
-                    loc=pred_samples_mean,
-                    scale=pred_samples_std,
-                    size=(self.samples, self.h, n_s),
-                ).reshape((self.h, -1))
         elif level == "groups":
             self.compute_error_for_every_group(error_metrics)
 
@@ -477,12 +366,6 @@ class CalculateResultsMint(CalculateResultsBase):
         error_metrics["mase"] = {}
         error_metrics["rmse"] = {}
         error_metrics["CRPS"] = {}
-        if self.store_prediction_points or self.store_prediction_samples:
-            error_metrics["predictions"] = {}
-        if self.store_prediction_points:
-            error_metrics["predictions"]["points"] = {}
-        if self.store_prediction_samples:
-            error_metrics["predictions"]["samples"] = {}
 
         error_metrics = self.mint_reconciliation("bottom", error_metrics)
         error_metrics = self.mint_reconciliation("total", error_metrics)
